/***************************************************************************
                          kcgame.cpp  -  description
                             -------------------
    begin                : Tue Dec 12 2000
    copyright            : (C) 2000 by Mathieu Jobin
    email                : somekool@mytradecenter.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcgame.h"

KCGame::KCGame(QWidget *parent)
: QWidget(parent, "KCGame main window"), kcMatrix(0)
{
	mw = parent;
	KCPosition::thegame = this;
	KCPlayer::m_game = this;
	config=KGlobal::config();
	readConfig ();
	kcpl = new KCPlayerList(this);
	KCnewGame();
}

KCGame::~KCGame()
{
	delete pLBr;
	delete pBar;
	delete pCtr;
	delete pBrd;
	delete kcMatrix;
}

void KCGame::resetMatrix()
{
//	long s = (3 + ((nbCols() - 1) * 4)) * (3 + ((nbRows() - 1) * 4));
	if (kcMatrix)
	{
		delete kcMatrix;
	}
	kcMatrix = new QValueList<long>();
}

/* this function reload the configuration and rebuild the window. */
void KCGame::KCrefresh()
{
	readConfig();
	KCResize();
	update();
}

void KCGame::KCnewGame ()
{
	resetMatrix();
	allPiece.setAutoDelete (true);
	allPiece.clear();
	allCompleted.setAutoDelete (true);
	allCompleted.clear();
	readConfig();
	KCResize();
	update();
}

void KCGame::readPlayerOptions ()
{
	kcpl->clear();
	kcpl->GenerePlayers();
}

void KCGame::readConfig()
{
	config->setGroup("Sizes");
	setNbCols (config->readNumEntry ("KCNbCols", _defNbCols));
	setNbRows (config->readNumEntry ("KCNbRows", _defNbRows));
	setSpacing (config->readNumEntry ("KCSpacing", _defSpacing));
	setBorder (config->readNumEntry ("KCBorder", _defBorder));
	setCtrWidth (config->readNumEntry ("KCNbCtrWidth", _defCtrWidth));
	setBarWidth (config->readNumEntry ("KCNbBarWidth", _setBarWidth), true);
	config->setGroup("Colors");
	pLBr = new QColor(config->readColorEntry ("KCLBrColor", new QColor(_defLBr)));
	pBar = new QColor(config->readColorEntry ("KCBarColor", new QColor(_defBar)));
	pCtr = new QColor(config->readColorEntry ("KCCtrColor", new QColor(_defCtr)));
	pBrd = new QColor(config->readColorEntry ("KCBrdColor", new QColor(_defBrd)));
	setPalette (QPalette (config->readColorEntry ("KCBGColor", new QColor(_defBGColor))));
}

//void KCGame::_buildGrid()
void KCGame::paintEvent (QPaintEvent *pe)
{
	QPainter p1, p2;
	KCPiece *kcp;
	QBrush b1 (*pCtr, SolidPattern);
	QBrush b2 (*pBrd, SolidPattern);
	QBrush b3 (*pLBr, SolidPattern);
	QBrush b4 (*pBar, SolidPattern);

	p1.begin (this);
	p2.begin (this);
	p1.setPen (*pCtr);
	p2.setPen (*pBrd);
	p1.setBrush (b1);
	p2.setBrush (b2);

	// displaying little circle.
	for (int y = 0; y < nbRows(); y++)
	{
		for (int x = 0; x < nbCols(); x++)
		{
			int CurX = Border() + (
				((Spacing() * 2) + CtrWidth() + BarWidth()) * x);
			int CurY = Border() + (
				((Spacing() * 2) + CtrWidth() + BarWidth()) * y);
			p2.drawEllipse (CurX, CurY, CtrWidth(), CtrWidth());
			p1.drawEllipse (CurX, CurY, (CtrWidth() - 1), (CtrWidth() - 1));
		}
	}
	p2.end();

	// displaying clicked bar.
	if (!allPiece.isEmpty())
	{
		p1.setPen (*pBar);
		p1.setBrush (b4);
		for (kcp=allPiece.first(); kcp != 0; kcp=allPiece.next())
		{
			p1.drawRect (kcp->m_x, kcp->m_y, kcp->m_w, kcp->m_h);
		}
		p1.setPen (*pLBr);
		p1.setBrush (b3);
		p1.drawRect (lastPiece->m_x, lastPiece->m_y, lastPiece->m_w, lastPiece->m_h);
	}

	// displaying completed square.
	if (!allCompleted.isEmpty())
	{
		QFont font("Helvetica", 24);
		p1.setFont(font);
		for (kcp=allCompleted.first(); kcp != 0; kcp=allCompleted.next()) // , i++)
		{
			p1.setPen (*pBrd);
			p1.setBrush (b2);
//			p1.drawRect (kcp->m_x, kcp->y, BarWidth(), BarWidth());
//			p1.setPen (*pCtr);
//			p1.setBrush (b1);
			p1.drawText(kcp->m_x + (Spacing()/2),
				kcp->m_y - (Spacing()/2) + BarWidth(), kcp->getPlayerLetter());
		}
	}
	p1.end();
	pe = NULL;
	//kdDebug () << i << endl;
}

void KCGame::MarkSquare (int a, int b, KCPlayer *p)
{
	KCCenterPiece *out;
int i;

	//kcMatrix->append(a + (b * nbCols()));
	kcMatrix->append(i = KCPosition(a, b).C());
	out = new KCCenterPiece();
	out->m_x = Id2Pos (a);
	out->m_y = Id2Pos (b);
	out->m_p = p;
	p->m_score++;
//	out->type = KCCENTERPIECE;
	allCompleted.append(out);
}

/* method KCGame::MoveIsPossible return true if the move is allowed.
	The move is allowed is the position isnt been clicked yet.
	the KCPosition t will be modified as require.
	if the function return false, another KCPosition should be tried. */
bool KCGame::MoveIsPossible(KCPosition *t) const
{
	bool retVal;
	int type;
	KCPosition r;

	retVal = false;
	type = KCPiece::pieceType (t->m_col, t->m_row);
	if (type == KCVERTICALPIECE || type == KCHORIZONTALPIECE)
	{
		if (!kcMatrix->contains(t->C()))
		{
			retVal = true;
		}
	}
	else
	{
		if (type == KCCENTERPIECE)
		{
			if (CanBeClose(*t, &r))
			{
				retVal = true;
				*t = r;
			}
		}
	}
	return retVal;
}

void KCGame::mouseReleaseEvent(QMouseEvent *e)
{
	int X, Y, IdX, IdY, type;
	bool Good, isntok;
	KCPiece *temp;
	KCPosition in, out;

	Good = false;
	isntok = false;
	X = Pixel2Pos(e->x(), &IdX);
	Y = Pixel2Pos(e->y(), &IdY);
	//kdDebug () << "x = " << e->x() << ", y = " << e->y() << ", X = " << X << ", Y = " << Y << ", IdX = " << IdX << ", IdY = " << IdY << endl;
	if (X < (3 + ((nbCols() - 1) * 4)) && Y < (3 + ((nbRows() - 1) * 4)))
	{
		type = KCPiece::pieceType (X, Y);
		in = KCPosition(X,Y);
		if (type == KCVERTICALPIECE || type == KCHORIZONTALPIECE)
		{
			if (MoveIsPossible(&in))
			{
				Good = true;
				temp = Pos2Piece (X, Y);
				//temp->m_p = kcpl->CurPlayer;
				MarkPlayed (temp);
				isntok = CheckFill (temp);
			}
		}
		else if (type == KCCENTERPIECE)
		{
			if (CanBeClose(in, &out))
			{
				Good = true;
				temp = Pos2Piece (out.m_col, out.m_row);
				MarkPlayed (temp);
				isntok = CheckFill (temp);
			}
		}
	}
	update();
	if (Good == true && false == isntok)
	{
		kcpl->Change_Player();
	}
}

void KCGame::mouseDoubleClickEvent (QMouseEvent *e)
{
	mouseReleaseEvent(e);
}

void KCGame::KCResize (void)
{
	long Width;
	long Height;
	int plus;
	
	config->setGroup("Sizes");
	plus = config->readNumEntry ("ToolBarHeight", 0) + config->readNumEntry ("StatusBarHeight", 0);

	Width = (BarWidth() * (nbCols() - 1)) + 
		(((Border() * 2) + CtrWidth()) * nbCols());
	Height = (BarWidth() * (nbRows() - 1)) + 
		(((Border() * 2) + CtrWidth()) * nbRows());
	static_cast<QWidget*>(parent())->setFixedSize (Width, Height + plus);// + 32);
	setFixedSize (Width, Height + plus);// + 32);
	update();
}

KCPosition KCGame::RandomizeSelectioningPiece() const
{
	KCPosition Index,
	           TempIndex;
	int Count;
	bool x, y;

	if (false == SquareCanBeClose(&TempIndex))
	{
		if (kcMatrix->count() + 25 > PiecePossible())
		{
			// i can improve that code...
			// first finding lowest and highest possible number in kcMatrix
			// then look only in free slots.
			// do it sequentialy...
			for (int i = 0; i < nbCols() * 4; i++)
			{
				for (int j = 0; j < nbRows() * 4; j++)
				{
					Index.m_col = i;
					Index.m_row = j;
					y = MoveIsPossible(&Index);
					if (y == true)
					{
						i = nbCols() * 4;
						j = nbRows() * 4;
					}
				}
			}
		}
		else {
		Count = 0;
		do
		{
			do
			{
				Index.m_col = 0;
				while (Index.m_col % 2 == 0)
					Index.m_col = entier(1, (nbCols() * 4) - 2);
				Index.m_row = 0;
				while (Index.m_row % 2 == 0)
					Index.m_row = entier(1, (nbRows() * 4) - 2);
				y = MoveIsPossible(&Index);
			} while (y == false);
			Count++;
			x = CouldBeClose(Index);// x = true;
		} while (x == true && Count != 25);
		kdDebug() << kcpl->CurPlayer->playerName() << " took " << Count << " tries." << endl;
		}
	}
	else
	{
		Index = TempIndex;
//		for I = 1 To (((My_Width_X + 1) * My_Height_Y) + ((My_Height_Y + 1) * My_Width_X))
//		{
//			if (TabPiece(I).Index = TempIndex)
//			{
//				Index = I;
//			}
//		}
	}
	return Index;
}

void KCGame::setNbCols (const int i, bool changed)
{
	iNbCols_ = i;
	if (changed)
	{
		KCResize();
	}
}

void KCGame::setNbRows (const int i, bool changed)
{
	iNbRows_ = i;
	if (changed)
	{
		KCResize();
	}
}

void KCGame::setSpacing (const int i, bool changed)
{
	iSpacing_ = i;
	if (changed)
	{
		KCResize();
	}
}

void KCGame::setBorder (const int i, bool changed)
{
	iBorder_ = i;
	if (changed)
	{
		KCResize();
	}
}

void KCGame::setBarWidth (const int i, bool changed)
{
	iBarWidth_ = i;
	if (changed)
	{
		KCResize();
	}
}

void KCGame::setBarHeight (const int i, bool changed)
{
	iBarHeight_ = i;
	if (changed)
	{
		KCResize();
	}
}

void KCGame::setCtrWidth (const int i, bool changed)
{
	iCtrWidth_ = i;
	if (changed)
	{
		KCResize();
	}
}

int KCGame::nbCols(void) const
{
	return iNbCols_;
}

int KCGame::nbRows(void) const
{
	return iNbRows_;
}

int KCGame::Spacing(void) const
{
	return iSpacing_;
}

int KCGame::Border(void) const
{
	return iBorder_;
}

int KCGame::BarWidth(void) const
{
	return iBarWidth_;
}

int KCGame::BarHeight(void) const
{
	return iBarHeight_;
}

int KCGame::CtrWidth(void) const
{
	return iCtrWidth_;
}

size_t KCGame::PiecePossible() const
{
	return SquarePossible() + BarPossible();
}

size_t KCGame::SquarePossible() const
{
	return ((nbCols() - 1) * nbRows()) + (nbCols() * (nbRows() - 1));
}

size_t KCGame::BarPossible() const
{
	return (nbCols() - 1) * (nbRows() - 1);
}


// i should think about a way to rewrite this function
// to get it better, it should search near the last move instead of
// scanning the whole board sequentially ;)
// also... maybe i should return the object or a NULL pointer in a false case
bool KCGame::SquareCanBeClose(KCPosition *ReturnValue) const
{
	ReturnValue->m_col = 0;
	ReturnValue->m_row = 0;
	for (int i = 0; i < (3 + ((nbCols() - 1) * 4)); i++)
	{
		for (int j = 0; j < (3 + ((nbRows() - 1) * 4)); j++)
		{
			if (KCPiece::pieceType (i, j) == KCCENTERPIECE)
			{
				if (CanBeClose(KCPosition(i, j), ReturnValue))
				{
					return true;
				}
			}
		}
	}
	return false;
}

// maybe should i change the prototype to ...
//KCPosition* KCGame::CanBeClose(KCPosition in) const
/* method KCGame::CanBeClose return true if the KCPosition is a KCCENTERPIECE
	and can be closed by clicking on a bar.
	if yes, the available bar is return as a the parameters KCPosition *out. */
bool KCGame::CanBeClose(KCPosition in, KCPosition *out) const
{
	int Count;

	Count = 0;
	if (KCPiece::pieceType(in.m_col, in.m_row) == KCCENTERPIECE)
	{
		if (kcMatrix->contains(in.N()))
		{
			Count = 1;
		}
		else
		{
			out->m_row = in.m_row - 2;
			out->m_col = in.m_col;
		}
		if (kcMatrix->contains(in.W()))
		{
			Count++;
		}
		else
		{
			out->m_row = in.m_row;
			out->m_col = in.m_col - 2;
		}
		if (kcMatrix->contains(in.E()))
		{
			Count++;
		}
		else
		{
			out->m_row = in.m_row;
			out->m_col = in.m_col + 2;
		}
		if (kcMatrix->contains(in.S()))
		{
			Count++;
		}
		else
		{
			out->m_row = in.m_row + 2;
			out->m_col = in.m_col;
		}
	}
	return Count == 3;
}

bool KCGame::CouldBeClose(const KCPosition &in) const
{
	bool Finish;
	int Count;

	Finish = false;
	switch (KCPiece::pieceType(in))
	{
		case KCVERTICALPIECE:
			Count = 0;
			if (kcMatrix->contains((in.m_col + 4) + ((in.m_row) * nbCols())))
			{
				Count = 1;
			}
			if (kcMatrix->contains(in.SE()))
			{
				Count++;
			}
			if (kcMatrix->contains(in.NE()))
			{
				Count++;
			}
			if (Count == 2)
			{
				Finish = true;
			}
			else
			{
				Count = 0;
				if (in.m_col > 0)
				{
					if (kcMatrix->contains((in.m_col - 4) + ((in.m_row) * nbCols())))
					{
						Count = 1;
					}
					if (kcMatrix->contains(in.SW()))
					{
						Count++;
					}
					if (kcMatrix->contains(in.NW()))
					{
						Count++;
					}
				}
			}
			if (Count == 2)
			{
				Finish = true;
			}
			break;
		case KCHORIZONTALPIECE:
			Count = 0;
			if (kcMatrix->contains(in.S(2)))
			{
				Count = 1;
			}
			if (kcMatrix->contains(in.SE()))
			{
				Count++;
			}
			if (kcMatrix->contains(in.SW()))
			{
				Count++;
			}
			if (Count == 2)
			{
				Finish = true;
			}
			else
			{
				Count = 0;
				if (in.m_row > 0)
				{
					if (kcMatrix->contains(in.N(2)))
					{
						Count = 1;
					}
					if (kcMatrix->contains(in.NE()))
					{
						Count++;
					}
					if (kcMatrix->contains(in.NW()))
					{
						Count++;
					}
				}
			}
			if (Count == 2)
			{
				Finish = true;
			}
			break;
		default:
			kdDebug () << "ERROR &" << endl;
			kapp->exit(1);
	}
	return Finish;
}

bool KCGame::IsGameOver() const
{
	if (kcMatrix->count() == PiecePossible())
		return true;
	else
		return false;
}

bool KCGame::CheckFill (KCPiece *temp)
{
	bool isntok;

	isntok = false;
	KCPieceType type = KCPiece::pieceType (*temp);
	if (type == KCHORIZONTALPIECE)
	{
		if (temp->m_pos->m_row > 0)
		{
			if ((kcMatrix->contains(temp->m_pos->NW()))
			&& (kcMatrix->contains(temp->m_pos->NE()))
			&& (kcMatrix->contains(temp->m_pos->N(2))))
			{
				if (!kcMatrix->contains(temp->m_pos->N()))
				{
					MarkSquare (temp->m_pos->m_col, temp->m_pos->m_row - 2, temp->m_p);
					isntok = true;
				}
			}
		}
		if (temp->m_pos->m_row <= (3 + ((nbRows() - 2) * 4)))
		{
			if ((kcMatrix->contains(temp->m_pos->SE()))
			&& (kcMatrix->contains(temp->m_pos->SW()))
			&& (kcMatrix->contains(temp->m_pos->S(2))))
			{
				if (!kcMatrix->contains(temp->m_pos->S()))
				{
					MarkSquare (temp->m_pos->m_col, temp->m_pos->m_row + 2, temp->m_p);
					isntok = true;
				}
			}
		}
	}
	else if (type == KCVERTICALPIECE)
	{
		if (temp->m_pos->m_col > 0)
		{
			if ((kcMatrix->contains(temp->m_pos->NW()))
			&& (kcMatrix->contains(temp->m_pos->SW()))
			&& (kcMatrix->contains(temp->m_pos->W(2))))
			{
				if (!kcMatrix->contains(temp->m_pos->W()))
				{
					MarkSquare (temp->m_pos->m_col - 2, temp->m_pos->m_row, temp->m_p);
					isntok = true;
				}
			}
		}
		if (temp->m_pos->m_col <= (3 + ((nbRows() - 2) * 4)))
		{
			if ((kcMatrix->contains(temp->m_pos->NE()))
			&& (kcMatrix->contains(temp->m_pos->SE()))
			&& (kcMatrix->contains(temp->m_pos->E(2))))
			{
				if (!kcMatrix->contains(temp->m_pos->E()))
				{
					MarkSquare (temp->m_pos->m_col + 2, temp->m_pos->m_row, temp->m_p);
					isntok = true;
				}
			}
		}
	}
	return isntok;
}

void KCGame::MarkPlayed (KCPiece *p)
{
	//int col_len = (3 + ((nbCols() - 1) * 4));
	int col_len = (nbCols() * 4) - 1;
	int row_len = (nbRows() * 4) - 1;

	if ((!isBusy(p)) && p->m_pos->m_col <= col_len && p->m_pos->m_row <= row_len)
	{
		allPiece.append(p);
		lastPiece = p;
		kcMatrix->append(KCPosition(*p).C());
		KCPlayer::Played = true;
		if (kcpl->CurPlayerType() == "KCComputerPlayer")
		{
			KCPlayer::ComputerHavePlayed = true;
		}
	}
}

KCPiece* KCGame::Pos2Piece (const KCPosition &p) const
{
	int IdX, IdY, type;
	KCPiece *temp;

	type = KCPiece::pieceType (p.m_col, p.m_row);
	IdX = Id2Pos(p.m_col);
	IdY = Id2Pos(p.m_row);

	if (type == KCHORIZONTALPIECE)
	{
		temp = new KCHorizontalPiece;
		temp->m_h = CtrWidth() / 3;
		temp->m_w = BarWidth();
		temp->m_x = IdX;
		temp->m_y = IdY + (CtrWidth() / 3);
	}
	else if (type == KCVERTICALPIECE)
	{
		temp = new KCVerticalPiece;
		temp->m_w = CtrWidth() / 3;
		temp->m_h = BarWidth();
		temp->m_x = IdX + (CtrWidth() / 3);
		temp->m_y = IdY;
	}
	temp->m_pos->m_col = p.m_col;
	temp->m_pos->m_row = p.m_row;
	temp->m_p = kcpl->CurPlayer;
	return temp;
}

KCPiece* KCGame::Pos2Piece (int col, int row) const
{
	return Pos2Piece (KCPosition(col,row));
}

int KCGame::Pixel2Pos (int x, int *Pos) const
{
	int i, CurW;

	CurW = Border ();
	*Pos = 0;
	i = 0;

	while ((x - *Pos) > CurW)
	{
		i++;
		*Pos += CurW;
		if ((i % 2) == 0)
		{
			// its a spacing
			CurW = Spacing();
		}
		else if ((i % 4) == 1)
		{
			// its a center
			CurW = CtrWidth();
		}
		else if ((i % 4) == 3)
		{
			// its a bar
			CurW = BarWidth();
		}
		else
		{
			// should never happen.
			kdDebug() << "ERROR _" << endl;
			kapp->exit(1);
		}
	}
	return i;
}

int KCGame::Id2Pos (int x) const
{
	int i, CurW, Pos;

	CurW = Border ();
	Pos = 0;
	i = 0;

	while (i != x)
	{
		i++;
		Pos += CurW;
		if ((i % 2) == 0)
		{
			// its a spacing
			CurW = Spacing();
		}
		else if ((i % 4) == 1)
		{
			// its a center
			CurW = CtrWidth();
		}
		else if ((i % 4) == 3)
		{
			// its a bar
			CurW = BarWidth();
		}
		else
		{
			// should never happen.
			kdDebug() << "ERROR +" << endl;
			kapp->exit(1);
		}
	}
	return Pos;
}

bool KCGame::isBusy (const KCPiece *kcpME)// const
{
	KCPiece *kcp;

	if (!allPiece.isEmpty())
	{
		// i have to use an iterator here.
		for (kcp=allPiece.first(); kcp != 0; kcp=allPiece.next())
		{
			if (kcpME->IsSame(*kcp, true))
			{
				return true;
			}
		}
	}
	return false;
}

#include "kcgame.moc"
