/***************************************************************************
                          kcgame.h  -  description
                             -------------------
    begin                : Tue Dec 12 2000
    copyright            : (C) 2000 by Mathieu Jobin
    email                : somekool@mytradecenter.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KCGAME_H
#define KCGAME_H

#include "kcdefault.h"
#include "kcpiece.h"
#include "kcplayer.h"
#include "kcplayerlist.h"
#include "utils.h"

class KCPlayer;
class KCPiece;
class KCPosition;
class KCPlayerList;

class KCGame : public QWidget
{
	Q_OBJECT

	public:
		KCGame(QWidget *parent = 0);
		~KCGame();
		void resetMatrix();
//		int pieceType (int X, int Y); /**/
		void MarkSquare (int a, int b, KCPlayer *p);
		bool MoveIsPossible(KCPosition *t) const;
		void Change_Player();
//		void PlayComputerTurn();
		KCPosition RandomizeSelectioningPiece() const;
//		bool SquareCanBeClose(KCPosition *ReturnValue);
//		bool isBusy (KCPiece *kcpME) /* KCPiece ? */;

		void setNbCols (const int i, bool changed = false);
		void setNbRows (const int i, bool changed = false);
		void setSpacing (const int i, bool changed = false);
		void setBorder (const int i, bool changed = false);
		void setBarWidth (const int i, bool changed = false);
		void setBarHeight (const int i, bool changed = false);
		void setCtrWidth (const int i, bool changed = false);
		int nbCols(void) const;
		int nbRows(void) const;
		int Spacing(void) const;
		int Border(void) const;
		int BarWidth(void) const;
		int BarHeight(void) const;
		int CtrWidth(void) const;

		size_t PiecePossible() const;
		size_t SquarePossible() const;
		size_t BarPossible() const;

		bool SquareCanBeClose(KCPosition *ReturnValue) const;
		bool CanBeClose(KCPosition in, KCPosition *out) const;
		bool CouldBeClose(const KCPosition &in) /* KCPiece ? */ const;
		bool IsGameOver() const;
		bool CheckFill (KCPiece *p);
		void MarkPlayed (KCPiece *p);
		bool isBusy (const KCPiece *kcpME);// const;
		KCPiece* Pos2Piece (const KCPosition &p) const;
		KCPiece* Pos2Piece (int col, int row) const;
		int Pixel2Pos (int x, int *Pos) const;
		int Id2Pos (int x) const;

	signals:
		void statusBarChanged(QString s);

	protected:
		void mouseReleaseEvent(QMouseEvent *e);
		virtual void mouseDoubleClickEvent(QMouseEvent *e);
		void paintEvent (QPaintEvent *pe);
		void readConfig ();

	public slots:
		void KCrefresh ();
		void KCResize (void);
		void KCnewGame ();
		void readPlayerOptions ();

	private:
		long iNbCols_;
		long iNbRows_;
		long iSpacing_;
		long iBorder_;
		long iBarWidth_;
		long iBarHeight_;
		long iCtrWidth_;
		QColor *pBar;
		QColor *pLBr;
		QColor *pCtr;
		QColor *pBrd;
		QWidget *mw;
		KCPiece *lastPiece;
		QPtrList<KCPiece> allPiece;
		QPtrList<KCPiece> allCompleted;
		QValueList<long>  *kcMatrix;
		KCPlayerList *kcpl;
		KConfig *config;

		void _initGame();
		//void _buildGrid();

		bool DrawGame;	// if its a draw game.
//		bool Played; 	// if the move is completed.
//		bool ComputerHavePlayed;	// Same, for the computer.
//		bool GameOver;
//		bool WasGameOver;
};

#endif //KCGAME_H

