/***************************************************************************
                          kcplayerlist.cpp  -  description
                             -------------------
    begin                : Thu Apr 11 2002
    copyright            : (C) 2002 by Mathieu Jobin
    email                : somekool@mytradecenter.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcplayerlist.h"

KCPlayerList::KCPlayerList (KCGame *opGame)
{
	PRECONDITION(opGame != 0);
	the_game = opGame;
	connect (this, SIGNAL(statusBarChanged(QString)), opGame->parent(), SLOT(slotRefreshStatusBar(QString)));
	Players = new QPtrList<KCPlayer>;
//	config = new KConfig;
	config=KGlobal::config();
	GenerePlayers();
	CurPlayer = Players->first();
//	CurPlayer = Players->at(0);
	Winner = 0;
	POSTCONDITION(the_game != 0);
}

KCPlayerList::~KCPlayerList ()
{
	clear();
	delete Players;
	CurPlayer = 0;
	Winner = 0;
	delete config;
}

void KCPlayerList::clear()
{
	while (!Players->isEmpty())
		Players->remove();
}

void KCPlayerList::GenerePlayers()
{
	int nbpl = 0, i = 0;

	while (!Players->isEmpty())
	{
		Players->removeLast();
	}
	config->setGroup("Game");
	nbpl = config->readNumEntry ("NbPlayers", 0);
	if (nbpl == 0)
	{
		AddPlayer(new KCHumanPlayer("MySelf"));
		AddPlayer(new KCComputerPlayer("Enemy", new QColor (240, 255, 255)));
//		AddAndSavePlayer(new KCComputerPlayer("Enemy2", new QColor (120, 255, 170)));
//		AddAndSavePlayer(new KCComputerPlayer("Enemy3", new QColor (0, 255, 255)));
	}
	else
	{
		for (i = 0; i < nbpl; i++)
		{
			QString *grp = new QString ("Player_");
			*grp += QString().number(i + 1);
			config->setGroup(grp->latin1());

			int type;
			QString *s = new QString (config->readEntry ("Name", QString("CROTTE")));
			QColor *c = new QColor(config->readColorEntry ("Color", new QColor(_defBrd)));
			type = config->readNumEntry ("Type", 0);
			switch (type)
			{
			case KCHUMAN:
				AddPlayer(new KCHumanPlayer (s, c));
				break;
			case KCCOMPUTER:
				AddPlayer(new KCComputerPlayer (s, c));
				break;
			case KCNETWORKED:
				AddPlayer(new KCNetworkedPlayer (s, c));
				break;
			}
			delete grp;
		}
	}
}

void KCPlayerList::ResetScore()
{
	KCPlayer *p;
	
	for (p=Players->first(); p != 0; p=Players->next())
	{
		p->m_score = 0;
	}
}

void KCPlayerList::SavePlayer(KCPlayer *p)
{
	QString *grp;
	
	grp = new QString ("Player_");
	*grp += QString().number(Players->count());
	config->setGroup(grp->latin1());
	delete grp;
	config->writeEntry("Name", p->m_name->latin1());
	if (p->className() == QString("KCHumanPlayer"))
	{
		config->writeEntry("Type", KCHUMAN);
	}
	else if (p->className() == QString("KCComputerPlayer"))
	{
		config->writeEntry("Type", KCCOMPUTER);
	}
	else if (p->className() == QString("KCNetworked"))
	{
		config->writeEntry("Type", KCNETWORKED);
	}
	config->writeEntry("Color", p->m_color->name());
	config->setGroup("Game");
	config->writeEntry("NbPlayers", Players->count());
}

/*void KCPlayerList::AddPlayer(QString *name, QColor *col, KCPlayer (*fn)())
{
	Players->append (new ((*fn)(name, col)));
}*/

void KCPlayerList::AddPlayer(KCPlayer *p)
{
	Players->append (p);
	connect (p, SIGNAL(havePlayed()), this, SLOT(Change_Player()));
	//AddPlayer(p->name, p->col, p->type);
	SavePlayer(p);
}

void KCPlayerList::DelPlayer(KCPlayer *p)
{
	Players->remove (p);
}

void KCPlayerList::DelPlayer(int i)
{
	Players->remove (i);
}

void KCPlayerList::DelPlayer(const QString &name)
{
	KCPlayer *a;

	for (a = Players->first(); a != 0; a = Players->next())
	{
		if (*(a->m_name) == name)
		{
			Players->remove(a);
		}
	}
}

int KCPlayerList::nbPlayer()
{
	KCPlayer *p;
	int i = 0;
	
	for (p=Players->first(); p != 0; p=Players->next())
	{i++;}
	return i;
}

QString KCPlayerList::CurPlayerType() const
{
	if (CurPlayer != 0)
		return CurPlayer->className();
	else
		return "";
}

void KCPlayerList::Change_Player()
{
	QString s = "Player ";
	int nbPlayers = Players->count();
	int curPlayerInd = Players->find(CurPlayer);

	KCPlayer::Played = false;
	if (curPlayerInd == (nbPlayers - 1))
	{
		CurPlayer = Players->first();
	}
	else
	{
		CurPlayer = Players->at(curPlayerInd + 1);
	}
	s += CurPlayer->playerName();
	s += "'s turn";
	kdDebug() << s << endl;
	emit statusBarChanged(s);
	if (KCPlayer::m_game->IsGameOver())
	{
		KCPlayer::WasGameOver = true;
		KCPlayer::GameOver = true;
		kdDebug() << "game over" << endl;
	}
	//static_cast<KloseCircuitApp*>(the_game->parent())->slotStatusMsg(CurPlayer->playerName());
//	if (CurPlayer->className() == QString("KCComputerPlayer"))
	{
		CurPlayer->PlayTurn();
	}
}

#include "kcplayerlist.moc"

