/***************************************************************************
                          klosecircuit.cpp  -  description
                             -------------------
    begin                : Thu Dec 20 2000
    copyright            : (C) 2000 by Mathieu Jobin
    email                : somekool@mytradecenter.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstatusbar.h>

// application specific includes
#include "klosecircuit.h"
#include "kcgame.h"
#include "kcconfdlg.h"

#define ID_STATUS_MSG 1
#define ID_STATUS_SCORE 2

KloseCircuitApp::KloseCircuitApp(QWidget* , const char* name):KMainWindow(0, name)
{
	config=kapp->config();
	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initStatusBar();
	initView();
	initActions();
	readOptions();

	///////////////////////////////////////////////////////////////////
	// disable actions at startup
	//fileNew->setEnabled(false);
	viewLongScore->setEnabled(false);
	AllowMouseChange->setEnabled(false);
}

KloseCircuitApp::~KloseCircuitApp()
{
}

void KloseCircuitApp::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	// STATUSBAR
	// TODO: add your own items you need for displaying current application status.
	statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
	statusBar()->insertItem(i18n("0000"), ID_STATUS_SCORE);
}

void KloseCircuitApp::initView()
{ 
	////////////////////////////////////////////////////////////////////
	// create the main widget here that is managed by KMainWindow's view-region and
	// connect the widget to your document to display document contents.

	view = new KCGame(this);
	setCentralWidget(view);	
//	connect(fileNew, SIGNAL( activated() ), view, SLOT( KCnewGame() ));
	connect(this, SIGNAL(sizeShouldChanged()), view, SLOT(KCResize()));
	connect(view, SIGNAL(statusBarChanged(QString)), this, SLOT(slotRefreshStatusBar(QString)));
}

void KloseCircuitApp::slotRefreshStatusBar (QString s)
{
	//statusBar()->insertItem(s), ID_STATUS_MSG);
	slotStatusMsg(s);
}

void KloseCircuitApp::initActions()
{
	fileNew = KStdAction::openNew(view, SLOT(KCnewGame()), actionCollection());
	fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
	viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
	viewLongScore = new KToggleAction(i18n("&Long Score"), 0, this, SLOT(slotViewLongScore()), actionCollection(), "LongScore");
	AllowMouseChange = new KToggleAction(i18n("&Mouse Change"), 0, this, SLOT(slotAllowMouseChange()), actionCollection(), "MouseChange");
	ShowSetupDialog = new KAction(i18n("&Setup"), 0, this, SLOT(slotShowSetupDialog()), actionCollection(), "SetupDlg");

	fileNew->setStatusText(i18n("Creates a new document"));
	fileQuit->setStatusText(i18n("Quits the application"));
	viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
	viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

	// use the absolute path to your klosecircuitui.rc file for testing purpose in createGUI();
	createGUI();
}

void KloseCircuitApp::saveOptions()
{	
  config->setGroup("General Options");
//  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar", viewStatusBar->isChecked());
//  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
}

void KloseCircuitApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
//  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

bool KloseCircuitApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KloseCircuitApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->next())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
//  slotStatusMsg(i18n("Ready."));
}

void KloseCircuitApp::slotViewToolBar()
{
	slotStatusMsg(i18n("Toggling toolbar..."));
	if(!viewToolBar->isChecked())
	{
		toolBar("mainToolBar")->hide();
	}
	else
	{
		toolBar("mainToolBar")->show();
	}		
	slotStatusMsg(i18n("Ready."));
}

void KloseCircuitApp::slotViewStatusBar()
{
	slotStatusMsg(i18n("Toggle the statusbar..."));
	if(!viewStatusBar->isChecked())
	{
		statusBar()->hide();
	}
	else
	{
		statusBar()->show();
	}
	slotStatusMsg(i18n("Ready."));
}

void KloseCircuitApp::slotViewLongScore()
{}

void KloseCircuitApp::slotAllowMouseChange()
{}

void KloseCircuitApp::slotShowSetupDialog()
{
	KCConfDlg *a;
	a = new KCConfDlg();
	connect(a, SIGNAL(configChanged()), view, SLOT(KCrefresh()));
	connect(a, SIGNAL(restartGame()), view, SLOT(KCnewGame()));
	connect(a, SIGNAL(PLconfigChanged()), view, SLOT(readPlayerOptions()));
	a->show();
}

void KloseCircuitApp::slotStatusMsg(const QString &text)
{
	statusBar()->clear();
	statusBar()->changeItem(text, ID_STATUS_MSG);
}

#include "klosecircuit.moc"
