/***************************************************************************
                          klosecircuit.h  -  description
                             -------------------
    begin                : Thu Dec 20 2000
    copyright            : (C) 2000 by Mathieu Jobin
    email                : somekool@mytradecenter.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLOSECIRCUIT_H
#define KLOSECIRCUIT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>

// forward declaration of the KloseCircuit classes
//class KCGame;

/**
  * The base class for KloseCircuit application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KloseCircuitView creates your center view, which is connected
  * to the window's Doc object.
  * KloseCircuitApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class KloseCircuitApp : public KMainWindow
{
  Q_OBJECT

  friend class KCGame;

  public:
    /** construtor of KloseCircuitApp, calls all init functions to create the application.
     */
    KloseCircuitApp(QWidget* parent=0, const char* name=0);
    ~KloseCircuitApp();

  signals:
   	void sizeShouldChanged ();

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */ 	
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();

  public slots:
	void slotRefreshStatusBar (QString s);
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();
    /** toggles the toolbar
     */
    void slotViewToolBar();
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    /** toggles the Long Score
     */
    void slotViewLongScore();
    /** toggles the Mouse Change
     */
    void slotAllowMouseChange();
    /** toggles the Long Score
     */
    void slotShowSetupDialog();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);

  private:
    /** the configuration object of the application */
    KConfig *config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    KCGame *view;

    // KAction pointers to enable/disable actions
    KAction* fileNew;
    KAction* fileQuit;
    KToggleAction* viewToolBar;
    KToggleAction* viewStatusBar;
    KToggleAction* viewLongScore;
    KToggleAction* AllowMouseChange;
    KAction* ShowSetupDialog;
};
 
#endif // KLOSECIRCUIT_H
